/* -------------------------------------------------------------------
   File "MagDyn_av_2D.pro"

    Magnetodynamics - Magnetic vector potential and electric scalar 
                      potential a-v formulation (2D)
   ------------------------------------------------------------------- 

   I N P U T
   ---------

   GlobalGroup :  (Extension '_Mag' is for Magnetic problem)
   -----------
   Domain_Mag               Whole magnetic domain
   DomainCC_Mag             Nonconducting regions (not used)
   DomainC_Mag              Conducting regions
   DomainS_Mag              Inductor regions (Source)
   DomainV_Mag              All regions in movement (for speed term)

   Function :
   --------
   nu[]                     Magnetic reluctivity
   sigma[]                  Electric conductivity

   Velocity[]               Velocity of regions

   Constraint :
   ----------
   MagneticVectorPotential_2D
                            Fixed magnetic vector potential (2D)
                            (classical boundary condition)
   SourceCurrentDensityZ    Fixed source current density (in Z direction)

   Voltage_2D               Fixed voltage
   Current_2D               Fixed Current

   Parameters :
   ----------

   Freq                     Frequency (Hz)

   Parameters for time loop with theta scheme :
   Mag_Time0, Mag_TimeMax, Mag_DTime
                            Initial time, Maximum time, Time step  (s)
   Mag_Theta                Theta  (e.g. 1.  : Implicit Euler,
                                         0.5 : Cranck Nicholson)
*/

Group {
  DefineGroup[ Domain_Mag, DomainCC_Mag, DomainM_Mag, DomainC_Mag,
               DomainS_Mag, DomainV_Mag ];
}

Function {
  DefineFunction[ nu, sigma ];
  DefineFunction[ Velocity ];
  DefineVariable[ Freq ];
  DefineVariable[ Mag_Time0, Mag_TimeMax, Mag_DTime, Mag_Theta ];
}

FunctionSpace {
  // Magnetic vector potential a (b = curl a)
  { Name Hcurl_a_Mag_2D; Type Form1P;
    BasisFunction {
      // a = a  s
      //      e  e
      { Name se; NameOfCoef ae; Function BF_PerpendicularEdge;
        Support Domain_Mag; Entity NodesOf[ All ]; }
    }
    Constraint {
      { NameOfCoef ae; EntityType NodesOf;
        NameOfConstraint MagneticVectorPotential_2D; }
    }
  }
  
  // Gradient of Electric scalar potential (2D)
  { Name Hregion_u_Mag_2D; Type Form1P;
    BasisFunction {
      { Name sr; NameOfCoef ur; Function BF_RegionZ;
        Support DomainE_Mag; Entity DomainE_Mag; }
    }
    GlobalQuantity {
      { Name U; Type AliasOf       ; NameOfCoef ur; }
      { Name I; Type AssociatedWith; NameOfCoef ur; }
    }
    Constraint {
      { NameOfCoef U; EntityType Region; NameOfConstraint Voltage_2D; }
      { NameOfCoef I; EntityType Region; NameOfConstraint Current_2D; }
    }
  } 
}


Formulation {
  { Name Magnetostatics_a_2D; Type FemEquation;
    Quantity {
      { Name a ; Type Local; NameOfSpace Hcurl_a_Mag_2D; }
    }
    Equation {
      Galerkin { [ nu[] * Dof{d a} , {d a} ]; In Domain_Mag;
                 Jacobian Vol; Integration CurlCurl; }
	  Galerkin { [ hc[], {d a} ]; In DomainM_Mag; 
	  			 Jacobian Vol; Integration CurlCurl; }
    }
  }
  { Name Magnetodynamics_av_2D; Type FemEquation;
    Quantity {  
      { Name a  ; Type Local  ; NameOfSpace Hcurl_a_Mag_2D ; }      
      { Name ur ; Type Local  ; NameOfSpace Hregion_u_Mag_2D ; }
    }

    Equation {
	  // PDE for magnetic field
      Galerkin { [ nu[] * Dof{d a} , {d a} ]; In Domain_Mag;  		// magnetic vector potential
                 Jacobian Vol; Integration CurlCurl; }
	  Galerkin { [ hc[], {d a} ]; In DomainM_Mag; 						// magnet sources
	  			 Jacobian Vol; Integration CurlCurl; }
	  			 
      // Electromagnetic coupling
      Galerkin { DtDof [ sigma[] * Dof{a} , {a} ]; In DomainE_Mag;		// induced field
                 Jacobian Vol; Integration CurlCurl; }
      Galerkin { [ sigma[] * Dof{ur} , {a} ]; In DomainE_Mag;			// voltage gradient
                 Jacobian Vol; Integration CurlCurl; }
	  Galerkin { [ sigma[] * ( velocity[] /\ Dof{d a} ) , {a} ]; In DomainV_Mag;
                 Jacobian Vol; Integration CurlCurl; }

	  // PDE for electric field
      Galerkin { DtDof [ sigma[] * Dof{a} , {ur} ]; In DomainE_Mag;		
                 Jacobian Vol; Integration CurlCurl; }
      Galerkin { [ sigma[] * Dof{ur} , {ur} ]; In DomainE_Mag;
                 Jacobian Vol; Integration CurlCurl; } 
	  Galerkin { [ sigma[] * ( velocity[] /\ Dof{d a} ) , {ur} ]; In DomainV_Mag;
                 Jacobian Vol; Integration CurlCurl; }  
    }
  }  
}


Resolution {
  { Name MagSta_a_2D;
    System {
      { Name ADyn; NameOfFormulation Magnetodynamics_av_2D; }
      { Name ASta; NameOfFormulation Magnetostatics_a_2D; DestinationSystem ADyn; }
    }
    Operation {
      InitSolution[ASta];
      Generate[ASta];
      Solve[ASta];
      SaveSolution[ASta];
      TransferSolution[ASta];
    }
  }
  
  { Name MagDyn_t_av_2D;
    System {
      { Name ADyn; NameOfFormulation Magnetodynamics_av_2D; }
    }
    Operation {
      InitSolution[ADyn]; 
      TimeLoopTheta[Mag_Time0, Mag_TimeMax, Mag_DTime, Mag_Theta] {
        Generate[ADyn];
		Solve[ADyn];
        SaveSolution[ADyn]; 
      }
    }
  }
}


PostProcessing {
  { Name MagDyn_av_2D; NameOfFormulation Magnetodynamics_av_2D;
    Quantity {
      { Name a; 
        Value { 
          Local { [ {a} ]; In Domain_Mag; Jacobian Vol; } 
        } 
      }
      { Name az; 
        Value { 
          Local { [ CompZ[{a}] ]; In Domain_Mag; Jacobian Vol; }
        }
      }
      { Name b; 
        Value { 
          Local { [ {d a} ]; In Domain_Mag; Jacobian Vol; }
        }
      }
      { Name j; 
        Value { 
          Local { [ - sigma[]*(Dt[{a}]+{ur}) ]; In DomainE_Mag; Jacobian Vol; } 
        } 
      } 	  
    }
  }    
}

